﻿using System.Numerics;

namespace Edge.Core.Domain.FccStationInfo.Input
{
    public class StationInfoInput
    {
        /// <summary>
        /// id
        /// </summary>
        public long? Id { get; set; }

        /// <summary>
        /// 云端站点 id
        /// </summary>
        public string? BuildId { get; set; }

        /// <summary>
        /// 站点名称
        /// </summary>
        public string? Name { get; set; }

        /// <summary>
        /// 经度
        /// </summary>
        public decimal? Longitude { get; set; }

        /// <summary>
        /// 纬度
        /// </summary>
        public decimal? Latitude { get; set; }

        /// <summary>
        /// 小程序链接地址
        /// </summary>
        public string? SmallProgram { get; set; }

        /// <summary>
        /// 云端地址
        /// </summary>
        public string? CloudService { get; set; }

        /// <summary>
        /// 云端  MQTT 地址
        /// </summary>
        public string? MqttService { get; set; }

        /// <summary>
        /// icard 数据库连接地址
        /// </summary>
        public string? IcardService { get; set; }

        /// <summary>
        /// 本地 fcc 与油机 socket 通讯端口
        /// </summary>
        public string? ServicePort { get; set; }

        /// <summary>
        /// 本地 fcc 与 fcc 页面 webSocket 端口
        /// </summary>
        public string? WebSocketPort { get; set; }

        public FccStationInfo? ToComponent()
        {
            if (IsNotNorm()) return null;

            return new FccStationInfo()
            {
                BuildId = BuildId,
                Name = Name != null ? Name : "",
                Longitude = (decimal)(Longitude != null ? Longitude : new decimal(0)),
                Latitude = (decimal)(Latitude != null ? Latitude : new decimal(0)),
                SmallProgram = SmallProgram,
                CloudService = CloudService,
                MqttService = MqttService,
                IcardService = IcardService != null ? IcardService : "",
                ServicePort = ServicePort,
                WebSocketPort = WebSocketPort
            };
        }

        public bool IsNotNorm()
        {
            return string.IsNullOrEmpty(BuildId) || !Longitude.HasValue || !Latitude.HasValue || string.IsNullOrEmpty(SmallProgram) || string.IsNullOrEmpty(CloudService)
                || string.IsNullOrEmpty(MqttService) || string.IsNullOrEmpty(ServicePort) || string.IsNullOrEmpty(WebSocketPort);
        }
    }
}
